@@ -24,8 +24,8 @@
 --
 --   Note that some functions like 'replicate', 'generate', or 'take', don't use
 --   the length of the list as argument, but the exponent of its length expressed
---   as a power of two. Throughout this document, this is referred (perhaps improperly)
---   as the /length index/. For example, if the list has length 16, its length index
+--   as a power of two. Throughout this document, this is referred
+--   as the /length exponent/. For example, if the list has length 16, its length exponent
 --   is 4 since 2^4 = 16. Therefore @replicate 4 0@ will create a list with 16 zeroes.
 --   Keep this in mind when using this library. Note as well that this implies that
 --   there is no need to check that the length argument is or is not a power of two.
@@ -33,6 +33,7 @@
 module Data.BinaryList (
     -- * Type
     BinList
+  , Exponent
     -- * Construction
   , singleton
   , append
@@ -42,7 +43,7 @@ module Data.BinaryList (
   , generate
   , generateM
     -- * Queries
-  , lengthIndex
+  , lengthExponent
   , length
   , lookup
   , head
@@ -52,6 +53,7 @@ module Data.BinaryList (
   , take
   , takeEnd
     -- * Transformation
+  , replace
   , reverse
     -- * Tuples
   , joinPairs
@@ -63,6 +65,7 @@ module Data.BinaryList (
     -- ** From list
   , fromList
   , fromListWithDefault
+  , fromListSplit
     -- ** To list
   , toListFilter
   , toListSegment
@@ -85,10 +88,18 @@ import Control.Arrow ((***))
 import Data.Monoid (mappend)
 import Data.Foldable (Foldable (..),toList)
 import Data.Traversable (Traversable (..))
-import Control.Monad.Trans.State (StateT (..),evalStateT,evalState,get,modify)
+import Control.Monad.Trans.State
+  ( StateT (..)
+  , evalStateT ,evalState
+  , runState
+  , get ,modify )
 import Control.Monad.Trans.Class (lift)
 import Data.Functor.Identity (Identity (..))
 import Control.Applicative.PhantomState
+import Data.Word (Word8)
+
+-- | An exponent.
+type Exponent = Word8
 
 -- | /O(1)/. Build a list with a single element.
 singleton :: a -> BinList a
@@ -96,19 +107,19 @@ singleton = ListEnd
 
 -- | /O(1)/. Given a binary list @l@ with length @2^k@:
 --
--- > lengthIndex l = k
+-- > lengthExponent l = k
 --
-lengthIndex :: BinList a -> Int
-lengthIndex (ListNode n _ _) = n
-lengthIndex (ListEnd _) = 0
+lengthExponent :: BinList a -> Exponent
+lengthExponent (ListNode n _ _) = n
+lengthExponent (ListEnd _) = 0
 
 -- | /O(1)/. Number of elements in the list.
 length :: BinList a -> Int
-length = (2^) . lengthIndex
+length = (2^) . lengthExponent
 
 {-# RULES
        "Data.BinaryList: length equality"
-         forall xs ys . length xs == length ys = lengthIndex xs == lengthIndex ys
+         forall xs ys . length xs == length ys = lengthExponent xs == lengthExponent ys
   #-}
 
 -- | /O(log n)/. Lookup an element in the list by its index (starting from 0).
@@ -122,13 +133,28 @@ lookup (ListNode n l r) i =
 lookup (ListEnd x) 0 = Just x
 lookup _ _ = Nothing
 
+-- | /O(log n)/. Replace a single element in the list. If the index is
+--   out of range, returns the original list.
+replace :: Int -- ^ Index to look for
+        -> a   -- ^ Element to insert
+        -> BinList a -> BinList a
+replace i0 y = go i0
+  where
+    go i (ListNode n l r) =
+      let m = 2^(n-1)
+      in  if i < m
+             then ListNode n (go i l)          r
+             else ListNode n       l (go (i-m) r)
+    go 0 (ListEnd _) = ListEnd y
+    go _ e = e
+
 -- | /O(1)/. Append two binary lists. This is only possible
 --   if both lists have the same length. If this condition
 --   is not hold, 'Nothing' is returned.
 append :: BinList a -> BinList a -> Maybe (BinList a)
 append xs ys =
-  let i = lengthIndex xs
-  in  if i == lengthIndex ys
+  let i = lengthExponent xs
+  in  if i == lengthExponent ys
          then Just $ ListNode (i+1) xs ys
          else Nothing
 
@@ -140,18 +166,18 @@ split (ListNode _ l r) = Right (l,r)
 split (ListEnd x) = Left x
 
 -- | /O(log n)/. Calling @take n xs@ returns the first @min (2^n) (length xs)@ elements of @xs@.
-take :: Int -> BinList a -> BinList a
+take :: Exponent -> BinList a -> BinList a
 take k xs@(ListNode n l _) = if k >= n then xs else take k l
 take _ xs = xs
 
 -- | /O(log n)/. Calling @takeEnd n xs@ returns the last @min (2^n) (length xs)@ elements of @xs@.
-takeEnd :: Int -> BinList a -> BinList a
+takeEnd :: Exponent -> BinList a -> BinList a
 takeEnd k xs@(ListNode n _ r) = if k >= n then xs else takeEnd k r
 takeEnd _ xs = xs
 
 -- | Calling @replicateA n f@ builds a binary list collecting the results of
 --   executing @2^n@ times the applicative action @f@.
-replicateA :: Applicative f => Int -> f a -> f (BinList a)
+replicateA :: Applicative f => Exponent -> f a -> f (BinList a)
 replicateA n f = go n
   where
     go 0 = ListEnd <$> f
@@ -159,7 +185,7 @@ replicateA n f = go n
            in  ListNode <$> pure i <*> b <*> b
 
 -- | The same as 'replicateA', but the actions are executed in reversed order.
-replicateAR :: Applicative f => Int -> f a -> f (BinList a)
+replicateAR :: Applicative f => Exponent -> f a -> f (BinList a)
 replicateAR n = forwards . replicateA n . Backwards
 
 {-# RULES
@@ -174,7 +200,7 @@ replicateAR n = forwards . replicateA n . Backwards
 
 -- | /O(log n)/. Calling @replicate n x@ builds a binary list with
 --   @2^n@ occurences of @x@.
-replicate :: Int -> a -> BinList a
+replicate :: Exponent -> a -> BinList a
 replicate n = runIdentity . replicateA n . Identity
 
 {-# RULES
@@ -182,14 +208,14 @@ replicate n = runIdentity . replicateA n . Identity
          forall f n x . map f (replicate n x) = replicate n (f x)
   #-}
 
--- | /O(n)/. Build a binary list with the given length index (see 'lengthIndex')
+-- | /O(n)/. Build a binary list with the given length exponent (see 'lengthExponent')
 --   by applying a function to each index.
-generate :: Int -> (Int -> a) -> BinList a
+generate :: Exponent -> (Int -> a) -> BinList a
 generate l f = evalState (replicateA l $ fmap f get <* modify (+1)) 0
 
 -- | Like 'generate', but the generator function returns a value in a 'Monad'.
 --   Therefore, the result is as well contained in a 'Monad'.
-generateM :: (Applicative m, Monad m) => Int -> (Int -> m a) -> m (BinList a)
+generateM :: (Applicative m, Monad m) => Exponent -> (Int -> m a) -> m (BinList a)
 generateM l f = evalStateT (replicateA l $ (get >>= lift . f) <* modify (+1)) 0
 
 -- | /O(log n)/. Get the first element of a binary list.
@@ -202,10 +228,9 @@ last :: BinList a -> a
 last (ListNode _ _ r) = last r
 last (ListEnd x) = x
 
-{-# INLINE[2] reverse #-}
-
 -- | /O(n)/. Reverse a binary list.
 reverse :: BinList a -> BinList a
+{-# INLINE[2] reverse #-}
 reverse (ListNode n l r) = ListNode n (reverse r) (reverse l)
 reverse xs = xs
 
@@ -217,22 +242,20 @@ reverse xs = xs
 ------------------------------
 -- Transformations with tuples
 
-{-# INLINE[1] joinPairs #-}
-
 -- | /O(n)/. Transform a list of pairs into a flat list. The
 --   resulting list will have twice more elements than the
 --   original.
 joinPairs :: BinList (a,a) -> BinList a
+{-# INLINE[1] joinPairs #-}
 joinPairs (ListEnd (x,y)) = ListNode 1 (ListEnd x) (ListEnd y)
 joinPairs (ListNode n l r) = ListNode (n+1) (joinPairs l) (joinPairs r)
 
-{-# INLINE [1] disjoinPairs #-}
-
 -- | /O(n)/. Opposite transformation of 'joinPairs'. It halves
 --   the number of elements of the input. As a result, when
 --   applied to a binary list with a single element, it returns
 --   'Nothing'.
 disjoinPairs :: BinList a -> Maybe (BinList (a,a))
+{-# INLINE [1] disjoinPairs #-}
 disjoinPairs (ListEnd _) = Nothing
 disjoinPairs xs = Just $ disjoinPairsNodes xs
 
@@ -251,11 +274,10 @@ disjoinPairsNodes _ = error "disjoinPairsNodes: bug. Please, report this with an
          forall f xs . disjoinPairs (map f (joinPairs xs)) = Just (map (f *** f) xs)
   #-}
 
-{-# INLINE[0] pairBuilder #-}
-
 -- | Expression @pairBuilder f xs@ is equivalent to @joinPairs (map f xs)@, but does
 --   not build any intermediate structure. Used for rewriting rules.
 pairBuilder :: (a -> (b,b)) -> BinList a -> BinList b
+{-# INLINE[0] pairBuilder #-}
 pairBuilder f = go
   where
     go (ListEnd x) = let (a,b) = f x in ListNode 1 (ListEnd a) (ListEnd b)
@@ -319,16 +341,14 @@ zipWith f = go
                      ListNode n (goEquals l l') (goEquals r r')
     goEquals xs ys = ListEnd $ f (head xs) (head ys)
 
-{-# INLINE zip #-}
-
 -- | /O(n)/. Zip two binary lists in pairs.
 zip :: BinList a -> BinList b -> BinList (a,b)
+{-# INLINE zip #-}
 zip = zipWith (,)
 
-{-# INLINE[1] unzip #-}
-
 -- | /O(n)/. Unzip a binary list of pairs.
 unzip :: BinList (a,b) -> (BinList a, BinList b)
+{-# INLINE[1] unzip #-}
 unzip (ListEnd (x,y)) = (ListEnd x, ListEnd y)
 unzip (ListNode n l r) =
   let (la,lb) = unzip l
@@ -356,7 +376,7 @@ unzipMap f = go
 
 -- | /O(log n)/. Calculate the exponent of a positive integer number expressed
 --   as a power of two.
-exponentInBasisTwo :: Int -> Maybe Int
+exponentInBasisTwo :: Int -> Maybe Exponent
 exponentInBasisTwo 1 = Just 0
 exponentInBasisTwo n =
   if even n
@@ -375,13 +395,13 @@ fromList xs = fmap builder . exponentInBasisTwo $ Prelude.length xs
 -- /Note: This value is system dependent, since the type 'Int' varies in size/
 -- /from system to system./
 --
-lastExponentOfTwo :: Int
-lastExponentOfTwo = 8 * sizeOf (undefined :: Int) - 2
+lastExponentOfTwo :: Exponent
+lastExponentOfTwo = fromIntegral $ 8 * sizeOf (undefined :: Int) - 2
 
 -- | /O(1)/. Calculate the next power of two exponent, if there is any. It is possible
 --   to not find a next one since the type 'Int' is finite. If the input is
 --   already a power of two, its exponent is returned.
-nextExponentOfTwo :: Int -> Maybe Int
+nextExponentOfTwo :: Int -> Maybe Exponent
 nextExponentOfTwo n = find (\i -> n <= 2^i) [0 .. lastExponentOfTwo]
 
 -- | /O(n)/. Build a binary list from a linked list. If the input list
@@ -397,16 +417,31 @@ fromListWithDefault e xs =
         Just n ->
           evalState (replicateA n $ StateT $
              \ys -> pure $ case ys of
-                      (h:t) -> (h,t)
-                      [] -> (e,[])
+                      (h:t) -> (h,t )
+                      _     -> (e,[])
                ) xs
         _ -> error "[binary-list] fromListWithDefault: input list is too big."
 
-{-# INLINE toListFilter #-}
+-- | /O(n)/. Build a binary list from a linked list. It returns a binary list
+--   with length @2 ^ n@ (where @n@ is the supplied 'Int' argument), and
+--   the list of elements of the original list that were not used. If the
+--   input list is shorter than @2 ^ n@, a default element will be used to
+--   complete the binary list. This method for building binary lists is faster
+--   than both 'fromList' and 'fromListWithDefault'.
+fromListSplit :: a   -- ^ Default element
+              -> Exponent -- ^ Length exponent
+              -> [a] -- ^ Input list
+              -> (BinList a, [a])
+fromListSplit e n =
+  runState $ replicateA n $ StateT $
+    \xs -> pure $ case xs of
+                    (h:t) -> (h,t )
+                    _     -> (e,[])
 
 -- | /O(n)/. Create a list from the elements of a binary list matching a given
 --   condition.
 toListFilter :: (a -> Bool) -> BinList a -> [a]
+{-# INLINE toListFilter #-}
 toListFilter c = foldr (\x -> if c x then (x:) else id) []
 
 -- | /O(n)/. Create a list extracting a sublist of elements from a binary list.
@@ -414,19 +449,17 @@ toListSegment :: Int -> Int -> BinList a -> [a]
 {-# INLINE toListSegment #-}
 toListSegment s e xs = runPhantomState (traverseSegment (changeState . (:)) s e xs) []
 
-{-# INLINE traverseSegment #-}
-
 -- | Apply an applicative action to every element in a segment of a binary list, from left to right.
 traverseSegment :: Applicative f => (a -> f ()) -> Int -> Int -> BinList a -> f ()
+{-# INLINE traverseSegment #-}
 traverseSegment f s e xs
   | s > e = pure ()
   | e < 0 = pure ()
   | s >= length xs = pure ()
   | otherwise = traverseSegmentFromTo f (max 0 s) e xs
 
-{-# INLINE traverseSegmentFromTo #-}
-
 traverseSegmentFromTo :: Applicative f => (a -> f ()) -> Int -> Int -> BinList a -> f ()
+{-# INLINE traverseSegmentFromTo #-}
 traverseSegmentFromTo f = go
   where
     go s e (ListNode n l r) =
@@ -441,9 +474,8 @@ traverseSegmentFromTo f = go
                      else traverseSegmentFrom f s l *> traverseSegmentTo f (e - k) r
     go _ _ (ListEnd x) = f x
 
-{-# INLINE traverseSegmentFrom #-}
-
 traverseSegmentFrom :: Applicative f => (a -> f ()) -> Int -> BinList a -> f ()
+{-# INLINE traverseSegmentFrom #-}
 traverseSegmentFrom f = go
   where
     go s (ListNode n l r) =
@@ -456,9 +488,8 @@ traverseSegmentFrom f = go
              else go s l *> traverseFull f r
     go _ (ListEnd x) = f x
 
-{-# INLINE traverseSegmentTo #-}
-
 traverseSegmentTo :: Applicative f => (a -> f ()) -> Int -> BinList a -> f ()
+{-# INLINE traverseSegmentTo #-}
 traverseSegmentTo f = go
   where
     go e (ListNode n l r) =
@@ -471,9 +502,8 @@ traverseSegmentTo f = go
              else traverseFull f l *> go (e - k) r
     go _ (ListEnd x) = f x
 
-{-# INLINE traverseFull #-}
-
 traverseFull :: Applicative f => (a -> f ()) -> BinList a -> f ()
+{-# INLINE traverseFull #-}
 traverseFull f = go
   where
     go (ListEnd x) = f x
@@ -496,8 +526,8 @@ actually writing 'map'. We do this to use 'map' in rewriting rules.
 
 -}
 
-{-# INLINE[1] map #-}
 map :: (a -> b) -> BinList a -> BinList b
+{-# INLINE[1] map #-}
 map f = go
   where
     go (ListEnd x) = ListEnd (f x)
